"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildChangelogNotes = exports.buildPlugin = exports.buildStrategy = exports.getChangelogTypes = exports.getVersioningStrategyTypes = exports.getReleaserTypes = void 0;
const go_1 = require("./strategies/go");
const go_yoshi_1 = require("./strategies/go-yoshi");
const java_yoshi_1 = require("./strategies/java-yoshi");
const krm_blueprint_1 = require("./strategies/krm-blueprint");
const ocaml_1 = require("./strategies/ocaml");
const php_1 = require("./strategies/php");
const php_yoshi_1 = require("./strategies/php-yoshi");
const python_1 = require("./strategies/python");
const ruby_1 = require("./strategies/ruby");
const ruby_yoshi_1 = require("./strategies/ruby-yoshi");
const rust_1 = require("./strategies/rust");
const simple_1 = require("./strategies/simple");
const terraform_module_1 = require("./strategies/terraform-module");
const helm_1 = require("./strategies/helm");
const elixir_1 = require("./strategies/elixir");
const dart_1 = require("./strategies/dart");
const node_1 = require("./strategies/node");
const default_1 = require("./versioning-strategies/default");
const always_bump_patch_1 = require("./versioning-strategies/always-bump-patch");
const service_pack_1 = require("./versioning-strategies/service-pack");
const dependency_manifest_1 = require("./versioning-strategies/dependency-manifest");
const node_workspace_1 = require("./plugins/node-workspace");
const cargo_workspace_1 = require("./plugins/cargo-workspace");
const github_1 = require("./changelog-notes/github");
const default_2 = require("./changelog-notes/default");
// Factory shared by GitHub Action and CLI for creating Release PRs
// and GitHub Releases:
// add any new releasers you create to this type as well as the `releasers`
// object below.
const allReleaseTypes = [
    'dart',
    'elixir',
    'go',
    'go-yoshi',
    'helm',
    'java-backport',
    'java-bom',
    'java-lts',
    'java-yoshi',
    'krm-blueprint',
    'node',
    'ocaml',
    'php',
    'php-yoshi',
    'python',
    'ruby',
    'ruby-yoshi',
    'rust',
    'simple',
    'terraform-module',
];
const releasers = {
    go: options => new go_1.Go(options),
    'go-yoshi': options => new go_yoshi_1.GoYoshi(options),
    'java-yoshi': options => new java_yoshi_1.JavaYoshi(options),
    'krm-blueprint': options => new krm_blueprint_1.KRMBlueprint(options),
    node: options => new node_1.Node(options),
    ocaml: options => new ocaml_1.OCaml(options),
    php: options => new php_1.PHP(options),
    'php-yoshi': options => new php_yoshi_1.PHPYoshi(options),
    python: options => new python_1.Python(options),
    rust: options => new rust_1.Rust(options),
    'terraform-module': options => new terraform_module_1.TerraformModule(options),
    helm: options => new helm_1.Helm(options),
    elixir: options => new elixir_1.Elixir(options),
    dart: options => new dart_1.Dart(options),
};
function getReleaserTypes() {
    return allReleaseTypes;
}
exports.getReleaserTypes = getReleaserTypes;
function getVersioningStrategyTypes() {
    return allVersioningTypes;
}
exports.getVersioningStrategyTypes = getVersioningStrategyTypes;
function getChangelogTypes() {
    return allChangelogNotesTypes;
}
exports.getChangelogTypes = getChangelogTypes;
async function buildStrategy(options) {
    var _a;
    const targetBranch = (_a = options.targetBranch) !== null && _a !== void 0 ? _a : options.github.repository.defaultBranch;
    const versioningStrategy = buildVersioningStrategy({
        type: options.versioning,
        bumpMinorPreMajor: options.bumpMinorPreMajor,
        bumpPatchForMinorPreMajor: options.bumpPatchForMinorPreMajor,
    });
    const changelogNotes = buildChangelogNotes({
        type: options.changelogType || 'default',
        github: options.github,
        changelogSections: options.changelogSections,
    });
    const strategyOptions = {
        github: options.github,
        targetBranch,
        path: options.path,
        bumpMinorPreMajor: options.bumpMinorPreMajor,
        bumpPatchForMinorPreMajor: options.bumpPatchForMinorPreMajor,
        component: options.component,
        packageName: options.packageName,
        changelogPath: options.changelogPath,
        changelogSections: options.changelogSections,
        versioningStrategy,
        skipGitHubRelease: options.skipGithubRelease,
        releaseAs: options.releaseAs,
        includeComponentInTag: options.includeComponentInTag,
        changelogNotes,
        pullRequestTitlePattern: options.pullRequestTitlePattern,
        extraFiles: options.extraFiles,
        tagSeparator: options.tagSeparator,
    };
    switch (options.releaseType) {
        case 'ruby': {
            return new ruby_1.Ruby({
                ...strategyOptions,
                versionFile: options.versionFile,
            });
        }
        case 'ruby-yoshi': {
            return new ruby_yoshi_1.RubyYoshi({
                ...strategyOptions,
                versionFile: options.versionFile,
            });
        }
        case 'java-backport': {
            return new java_yoshi_1.JavaYoshi({
                ...strategyOptions,
                versioningStrategy: new always_bump_patch_1.AlwaysBumpPatch(),
            });
        }
        case 'java-bom': {
            return new java_yoshi_1.JavaYoshi({
                ...strategyOptions,
                versioningStrategy: new dependency_manifest_1.DependencyManifest({
                    bumpMinorPreMajor: options.bumpMinorPreMajor,
                    bumpPatchForMinorPreMajor: options.bumpPatchForMinorPreMajor,
                }),
            });
        }
        case 'java-lts': {
            return new java_yoshi_1.JavaYoshi({
                ...strategyOptions,
                versioningStrategy: new service_pack_1.ServicePackVersioningStrategy(),
            });
        }
        case 'simple': {
            return new simple_1.Simple({
                ...strategyOptions,
                versionFile: options.versionFile,
            });
        }
        default: {
            const builder = releasers[options.releaseType];
            if (builder) {
                return builder(strategyOptions);
            }
            throw new Error(`Unknown release type: ${options.releaseType}`);
        }
    }
}
exports.buildStrategy = buildStrategy;
const allVersioningTypes = [
    'default',
    'always-bump-patch',
    'service-pack',
];
function buildVersioningStrategy(options) {
    switch (options.type) {
        case 'always-bump-patch':
            return new always_bump_patch_1.AlwaysBumpPatch(options);
        case 'service-pack':
            return new service_pack_1.ServicePackVersioningStrategy(options);
        default:
            return new default_1.DefaultVersioningStrategy(options);
    }
}
function buildPlugin(options) {
    switch (options.type) {
        case 'cargo-workspace':
            return new cargo_workspace_1.CargoWorkspace(options.github, options.targetBranch, options.repositoryConfig, options);
        case 'node-workspace':
            return new node_workspace_1.NodeWorkspace(options.github, options.targetBranch, options.repositoryConfig, options);
        default:
            throw new Error(`Unknown plugin type: ${options.type}`);
    }
}
exports.buildPlugin = buildPlugin;
const allChangelogNotesTypes = ['default', 'github'];
function buildChangelogNotes(options) {
    switch (options.type) {
        case 'github':
            return new github_1.GitHubChangelogNotes(options.github);
        case 'default':
            return new default_2.DefaultChangelogNotes(options);
        default:
            throw new Error(`Unknown changelog type: ${options.type}`);
    }
}
exports.buildChangelogNotes = buildChangelogNotes;
//# sourceMappingURL=factory.js.map