"use strict";
// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Manifest = exports.MANIFEST_PULL_REQUEST_TITLE_PATTERN = exports.ROOT_PROJECT_PATH = exports.DEFAULT_RELEASE_PLEASE_MANIFEST = exports.DEFAULT_RELEASE_PLEASE_CONFIG = void 0;
const version_1 = require("./version");
const logger_1 = require("./util/logger");
const commit_split_1 = require("./util/commit-split");
const tag_name_1 = require("./util/tag-name");
const branch_name_1 = require("./util/branch-name");
const pull_request_title_1 = require("./util/pull-request-title");
const factory_1 = require("./factory");
const pull_request_body_1 = require("./util/pull-request-body");
const merge_1 = require("./plugins/merge");
const release_please_manifest_1 = require("./updaters/release-please-manifest");
const errors_1 = require("./errors");
exports.DEFAULT_RELEASE_PLEASE_CONFIG = 'release-please-config.json';
exports.DEFAULT_RELEASE_PLEASE_MANIFEST = '.release-please-manifest.json';
exports.ROOT_PROJECT_PATH = '.';
const DEFAULT_COMPONENT_NAME = '';
const DEFAULT_LABELS = ['autorelease: pending'];
const DEFAULT_RELEASE_LABELS = ['autorelease: tagged'];
exports.MANIFEST_PULL_REQUEST_TITLE_PATTERN = 'chore: release ${branch}';
class Manifest {
    /**
     * Create a Manifest from explicit config in code. This assumes that the
     * repository has a single component at the root path.
     *
     * @param {GitHub} github GitHub client
     * @param {string} targetBranch The releaseable base branch
     * @param {RepositoryConfig} repositoryConfig Parsed configuration of path => release configuration
     * @param {ReleasedVersions} releasedVersions Parsed versions of path => latest release version
     * @param {ManifestOptions} manifestOptions Optional. Manifest options
     * @param {string} manifestOptions.bootstrapSha If provided, use this SHA
     *   as the point to consider commits after
     * @param {boolean} manifestOptions.alwaysLinkLocal Option for the node-workspace
     *   plugin
     * @param {boolean} manifestOptions.separatePullRequests If true, create separate pull
     *   requests instead of a single manifest release pull request
     * @param {PluginType[]} manifestOptions.plugins Any plugins to use for this repository
     * @param {boolean} manifestOptions.fork If true, create pull requests from a fork. Defaults
     *   to `false`
     * @param {string} manifestOptions.signoff Add a Signed-off-by annotation to the commit
     * @param {string} manifestOptions.manifestPath Path to the versions manifest
     * @param {string[]} manifestOptions.labels Labels that denote a pending, untagged release
     *   pull request. Defaults to `[autorelease: pending]`
     * @param {string[]} manifestOptions.releaseLabels Labels to apply to a tagged release
     *   pull request. Defaults to `[autorelease: tagged]`
     */
    constructor(github, targetBranch, repositoryConfig, releasedVersions, manifestOptions) {
        var _a;
        this.repository = github.repository;
        this.github = github;
        this.targetBranch = targetBranch;
        this.repositoryConfig = repositoryConfig;
        this.releasedVersions = releasedVersions;
        this.manifestPath =
            (manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.manifestPath) || exports.DEFAULT_RELEASE_PLEASE_MANIFEST;
        this.separatePullRequests = (_a = manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.separatePullRequests) !== null && _a !== void 0 ? _a : Object.keys(repositoryConfig).length === 1;
        this.plugins = (manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.plugins) || [];
        this.fork = (manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.fork) || false;
        this.signoffUser = manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.signoff;
        this.releaseLabels =
            (manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.releaseLabels) || DEFAULT_RELEASE_LABELS;
        this.labels = (manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.labels) || DEFAULT_LABELS;
        this.bootstrapSha = manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.bootstrapSha;
        this.lastReleaseSha = manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.lastReleaseSha;
        this.draft = manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.draft;
        this.draftPullRequest = manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.draftPullRequest;
        this.groupPullRequestTitlePattern = manifestOptions === null || manifestOptions === void 0 ? void 0 : manifestOptions.groupPullRequestTitlePattern;
    }
    /**
     * Create a Manifest from config files in the repository.
     *
     * @param {GitHub} github GitHub client
     * @param {string} targetBranch The releaseable base branch
     * @param {string} configFile Optional. The path to the manifest config file
     * @param {string} manifestFile Optional. The path to the manifest versions file
     * @returns {Manifest}
     */
    static async fromManifest(github, targetBranch, configFile = exports.DEFAULT_RELEASE_PLEASE_CONFIG, manifestFile = exports.DEFAULT_RELEASE_PLEASE_MANIFEST, manifestOptionOverrides = {}) {
        const [{ config: repositoryConfig, options: manifestOptions }, releasedVersions,] = await Promise.all([
            parseConfig(github, configFile, targetBranch),
            parseReleasedVersions(github, manifestFile, targetBranch),
        ]);
        return new Manifest(github, targetBranch, repositoryConfig, releasedVersions, {
            manifestPath: manifestFile,
            ...manifestOptions,
            ...manifestOptionOverrides,
        });
    }
    /**
     * Create a Manifest from explicit config in code. This assumes that the
     * repository has a single component at the root path.
     *
     * @param {GitHub} github GitHub client
     * @param {string} targetBranch The releaseable base branch
     * @param {ReleaserConfig} config Release strategy options
     * @param {ManifestOptions} manifestOptions Optional. Manifest options
     * @param {string} manifestOptions.bootstrapSha If provided, use this SHA
     *   as the point to consider commits after
     * @param {boolean} manifestOptions.alwaysLinkLocal Option for the node-workspace
     *   plugin
     * @param {boolean} manifestOptions.separatePullRequests If true, create separate pull
     *   requests instead of a single manifest release pull request
     * @param {PluginType[]} manifestOptions.plugins Any plugins to use for this repository
     * @param {boolean} manifestOptions.fork If true, create pull requests from a fork. Defaults
     *   to `false`
     * @param {string} manifestOptions.signoff Add a Signed-off-by annotation to the commit
     * @param {string} manifestOptions.manifestPath Path to the versions manifest
     * @param {string[]} manifestOptions.labels Labels that denote a pending, untagged release
     *   pull request. Defaults to `[autorelease: pending]`
     * @param {string[]} manifestOptions.releaseLabels Labels to apply to a tagged release
     *   pull request. Defaults to `[autorelease: tagged]`
     * @returns {Manifest}
     */
    static async fromConfig(github, targetBranch, config, manifestOptions, path = exports.ROOT_PROJECT_PATH) {
        const repositoryConfig = {};
        repositoryConfig[path] = config;
        const strategy = await factory_1.buildStrategy({
            github,
            ...config,
        });
        const component = await strategy.getComponent();
        const releasedVersions = {};
        const latestVersion = await latestReleaseVersion(github, targetBranch, config.includeComponentInTag ? component : '', config.pullRequestTitlePattern);
        if (latestVersion) {
            releasedVersions[path] = latestVersion;
        }
        return new Manifest(github, targetBranch, repositoryConfig, releasedVersions, {
            separatePullRequests: true,
            ...manifestOptions,
        });
    }
    /**
     * Build all candidate pull requests for this repository.
     *
     * Iterates through each path and builds a candidate pull request for component.
     * Applies any configured plugins.
     *
     * @returns {ReleasePullRequest[]} The candidate pull requests to open or update.
     */
    async buildPullRequests() {
        var _a;
        logger_1.logger.info('Building pull requests');
        const pathsByComponent = await this.getPathsByComponent();
        const strategiesByPath = await this.getStrategiesByPath();
        // Collect all the SHAs of the latest release packages
        logger_1.logger.info('Collecting release commit SHAs');
        let releasesFound = 0;
        const expectedReleases = Object.keys(strategiesByPath).length;
        // SHAs by path
        const releaseShasByPath = {};
        // Releases by path
        const releasesByPath = {};
        for await (const release of this.github.releaseIterator({
            maxResults: 400,
        })) {
            const tagName = tag_name_1.TagName.parse(release.tagName);
            if (!tagName) {
                logger_1.logger.warn(`Unable to parse release name: ${release.name}`);
                continue;
            }
            const component = tagName.component || DEFAULT_COMPONENT_NAME;
            const path = pathsByComponent[component];
            if (!path) {
                logger_1.logger.warn(`Found release tag with component '${component}', but not configured in manifest`);
                continue;
            }
            const expectedVersion = this.releasedVersions[path];
            if (!expectedVersion) {
                logger_1.logger.warn(`Unable to find expected version for path '${path}' in manifest`);
                continue;
            }
            if (expectedVersion.toString() === tagName.version.toString()) {
                logger_1.logger.debug(`Found release for path ${path}, ${release.tagName}`);
                releaseShasByPath[path] = release.sha;
                releasesByPath[path] = {
                    name: release.name,
                    tag: tagName,
                    sha: release.sha,
                    notes: release.notes || '',
                };
                releasesFound += 1;
            }
            if (releasesFound >= expectedReleases) {
                break;
            }
        }
        const needsBootstrap = releasesFound < expectedReleases;
        if (releasesFound < expectedReleases) {
            logger_1.logger.warn(`Expected ${expectedReleases} releases, only found ${releasesFound}`);
            // Fall back to looking for missing releases using expected tags
            const missingPaths = Object.keys(strategiesByPath).filter(path => !releasesByPath[path]);
            logger_1.logger.warn(`Missing ${missingPaths.length} paths: ${missingPaths}`);
            const missingReleases = await this.backfillReleasesFromTags(missingPaths, strategiesByPath);
            for (const path in missingReleases) {
                releaseShasByPath[path] = missingReleases[path].sha;
                releasesByPath[path] = missingReleases[path];
                releasesFound++;
            }
        }
        if (releasesFound < expectedReleases) {
            logger_1.logger.warn(`Expected ${expectedReleases} releases, only found ${releasesFound}`);
        }
        for (const path in releasesByPath) {
            const release = releasesByPath[path];
            logger_1.logger.debug(`release for path: ${path}, version: ${release.tag.version.toString()}, sha: ${release.sha}`);
        }
        // iterate through commits and collect commits until we have
        // seen all release commits
        logger_1.logger.info('Collecting commits since all latest releases');
        const commits = [];
        const commitGenerator = this.github.mergeCommitIterator(this.targetBranch, {
            maxResults: 500,
            backfillFiles: true,
        });
        const releaseShas = new Set(Object.values(releaseShasByPath));
        logger_1.logger.debug(releaseShas);
        const expectedShas = releaseShas.size;
        // sha => release pull request
        const releasePullRequestsBySha = {};
        let releaseCommitsFound = 0;
        for await (const commit of commitGenerator) {
            if (releaseShas.has(commit.sha)) {
                if (commit.pullRequest) {
                    releasePullRequestsBySha[commit.sha] = commit.pullRequest;
                }
                else {
                    logger_1.logger.warn(`Release SHA ${commit.sha} did not have an associated pull request`);
                }
                releaseCommitsFound += 1;
            }
            if (this.lastReleaseSha && this.lastReleaseSha === commit.sha) {
                logger_1.logger.info(`Using configured lastReleaseSha ${this.lastReleaseSha} as last commit.`);
                break;
            }
            else if (needsBootstrap && commit.sha === this.bootstrapSha) {
                logger_1.logger.info(`Needed bootstrapping, found configured bootstrapSha ${this.bootstrapSha}`);
                break;
            }
            else if (!needsBootstrap && releaseCommitsFound >= expectedShas) {
                // found enough commits
                break;
            }
            commits.push({
                sha: commit.sha,
                message: commit.message,
                files: commit.files,
                pullRequest: commit.pullRequest,
            });
        }
        if (releaseCommitsFound < expectedShas) {
            logger_1.logger.warn(`Expected ${expectedShas} commits, only found ${releaseCommitsFound}`);
        }
        // split commits by path
        logger_1.logger.info(`Splitting ${commits.length} commits by path`);
        const cs = new commit_split_1.CommitSplit({
            includeEmpty: true,
            packagePaths: Object.keys(this.repositoryConfig),
        });
        const commitsPerPath = cs.split(commits);
        let newReleasePullRequests = [];
        for (const path in this.repositoryConfig) {
            const config = this.repositoryConfig[path];
            logger_1.logger.info(`Building candidate release pull request for path: ${path}`);
            logger_1.logger.debug(`type: ${config.releaseType}`);
            logger_1.logger.debug(`targetBranch: ${this.targetBranch}`);
            const pathCommits = commitsAfterSha(path === exports.ROOT_PROJECT_PATH ? commits : commitsPerPath[path], releaseShasByPath[path]);
            logger_1.logger.debug(`commits: ${pathCommits.length}`);
            const latestReleasePullRequest = releasePullRequestsBySha[releaseShasByPath[path]];
            if (!latestReleasePullRequest) {
                logger_1.logger.warn('No latest release pull request found.');
            }
            const strategy = strategiesByPath[path];
            let latestRelease = releasesByPath[path];
            if (!latestRelease &&
                this.releasedVersions[path] &&
                this.releasedVersions[path].toString() !== '0.0.0') {
                const version = this.releasedVersions[path];
                const component = await strategy.getComponent();
                logger_1.logger.info(`No latest release found for path: ${path}, component: ${component}, but a previous version (${version.toString()}) was specified in the manifest.`);
                latestRelease = {
                    tag: new tag_name_1.TagName(version, component),
                    sha: '',
                    notes: '',
                };
            }
            const releasePullRequest = await strategy.buildReleasePullRequest(pathCommits, latestRelease, (_a = config.draftPullRequest) !== null && _a !== void 0 ? _a : this.draftPullRequest, this.labels);
            if (releasePullRequest) {
                if (releasePullRequest.version) {
                    const versionsMap = new Map();
                    versionsMap.set(path, releasePullRequest.version);
                    releasePullRequest.updates.push({
                        path: this.manifestPath,
                        createIfMissing: false,
                        updater: new release_please_manifest_1.ReleasePleaseManifest({
                            version: releasePullRequest.version,
                            versionsMap,
                        }),
                    });
                }
                newReleasePullRequests.push({
                    path,
                    config,
                    pullRequest: releasePullRequest,
                });
            }
        }
        // Build plugins
        const plugins = this.plugins.map(pluginType => factory_1.buildPlugin({
            type: pluginType,
            github: this.github,
            targetBranch: this.targetBranch,
            repositoryConfig: this.repositoryConfig,
        }));
        // Combine pull requests into 1 unless configured for separate
        // pull requests
        if (!this.separatePullRequests) {
            plugins.push(new merge_1.Merge(this.github, this.targetBranch, this.repositoryConfig, this.groupPullRequestTitlePattern));
        }
        for (const plugin of plugins) {
            newReleasePullRequests = await plugin.run(newReleasePullRequests);
        }
        return newReleasePullRequests.map(pullRequestWithConfig => pullRequestWithConfig.pullRequest);
    }
    async backfillReleasesFromTags(missingPaths, strategiesByPath) {
        const releasesByPath = {};
        const allTags = await this.getAllTags();
        for (const path of missingPaths) {
            const expectedVersion = this.releasedVersions[path];
            if (!expectedVersion) {
                logger_1.logger.warn(`No version for path ${path}`);
                continue;
            }
            const component = await strategiesByPath[path].getComponent();
            const expectedTag = new tag_name_1.TagName(expectedVersion, component, this.repositoryConfig[path].tagSeparator);
            logger_1.logger.debug(`looking for tagName: ${expectedTag.toString()}`);
            const foundTag = allTags[expectedTag.toString()];
            if (foundTag) {
                logger_1.logger.debug(`found: ${foundTag.name} ${foundTag.sha}`);
                releasesByPath[path] = {
                    name: foundTag.name,
                    tag: expectedTag,
                    sha: foundTag.sha,
                    notes: '',
                };
            }
        }
        return releasesByPath;
    }
    async getAllTags() {
        const allTags = {};
        for await (const tag of this.github.tagIterator()) {
            allTags[tag.name] = tag;
        }
        return allTags;
    }
    /**
     * Opens/updates all candidate release pull requests for this repository.
     *
     * @returns {number[]} Pull request numbers of release pull requests
     */
    async createPullRequests() {
        const candidatePullRequests = await this.buildPullRequests();
        if (candidatePullRequests.length === 0) {
            return [];
        }
        // if there are any merged, pending release pull requests, don't open
        // any new release PRs
        const mergedPullRequestsGenerator = this.findMergedReleasePullRequests();
        for await (const _ of mergedPullRequestsGenerator) {
            logger_1.logger.warn('There are untagged, merged release PRs outstanding - aborting');
            return [];
        }
        // collect open release pull requests
        logger_1.logger.info('Looking for open release pull requests');
        const openPullRequests = [];
        const generator = this.github.pullRequestIterator(this.targetBranch, 'OPEN');
        for await (const openPullRequest of generator) {
            if (hasAllLabels(this.labels, openPullRequest.labels) &&
                branch_name_1.BranchName.parse(openPullRequest.headBranchName) &&
                pull_request_body_1.PullRequestBody.parse(openPullRequest.body)) {
                openPullRequests.push(openPullRequest);
            }
        }
        logger_1.logger.info(`found ${openPullRequests.length} open release pull requests.`);
        const promises = [];
        for (const pullRequest of candidatePullRequests) {
            promises.push(this.createOrUpdatePullRequest(pullRequest, openPullRequests));
        }
        return await Promise.all(promises);
    }
    async createOrUpdatePullRequest(pullRequest, openPullRequests) {
        // look for existing, open pull rquest
        const existing = openPullRequests.find(openPullRequest => openPullRequest.headBranchName === pullRequest.headRefName);
        if (existing) {
            // If unchanged, no need to push updates
            if (existing.body === pullRequest.body.toString()) {
                logger_1.logger.info(`PR https://github.com/${this.repository.owner}/${this.repository.repo}/pull/${existing.number} remained the same`);
                return undefined;
            }
            const updatedPullRequest = await this.github.updatePullRequest(existing.number, pullRequest, this.targetBranch, {
                fork: this.fork,
                signoffUser: this.signoffUser,
            });
            return updatedPullRequest;
        }
        else {
            const newPullRequest = await this.github.createReleasePullRequest(pullRequest, this.targetBranch, {
                fork: this.fork,
                signoffUser: this.signoffUser,
            });
            return newPullRequest;
        }
    }
    async *findMergedReleasePullRequests() {
        // Find merged release pull requests
        const pullRequestGenerator = this.github.pullRequestIterator(this.targetBranch, 'MERGED', 200);
        for await (const pullRequest of pullRequestGenerator) {
            if (!hasAllLabels(this.labels, pullRequest.labels)) {
                continue;
            }
            logger_1.logger.debug(`Found pull request #${pullRequest.number}: '${pullRequest.title}'`);
            const pullRequestBody = pull_request_body_1.PullRequestBody.parse(pullRequest.body);
            if (!pullRequestBody) {
                logger_1.logger.debug('could not parse pull request body as a release PR');
                continue;
            }
            yield pullRequest;
        }
    }
    /**
     * Find merged, untagged releases and build candidate releases to tag.
     *
     * @returns {CandidateRelease[]} List of release candidates
     */
    async buildReleases() {
        var _a;
        logger_1.logger.info('Building releases');
        const strategiesByPath = await this.getStrategiesByPath();
        // Find merged release pull requests
        const generator = await this.findMergedReleasePullRequests();
        const releases = [];
        for await (const pullRequest of generator) {
            for (const path in this.repositoryConfig) {
                const config = this.repositoryConfig[path];
                logger_1.logger.info(`Building release for path: ${path}`);
                logger_1.logger.debug(`type: ${config.releaseType}`);
                logger_1.logger.debug(`targetBranch: ${this.targetBranch}`);
                const strategy = strategiesByPath[path];
                const release = await strategy.buildRelease(pullRequest);
                if (release) {
                    releases.push({
                        ...release,
                        path,
                        pullRequest,
                        draft: (_a = config.draft) !== null && _a !== void 0 ? _a : this.draft,
                        prerelease: config.prerelease &&
                            (!!release.tag.version.preRelease ||
                                release.tag.version.major === 0),
                    });
                }
                else {
                    logger_1.logger.info(`No release necessary for path: ${path}`);
                }
            }
        }
        return releases;
    }
    /**
     * Find merged, untagged releases. For each release, create a GitHub release,
     * comment on the pull request used to generated it and update the pull request
     * labels.
     *
     * @returns {GitHubRelease[]} List of created GitHub releases
     */
    async createReleases() {
        const releasesByPullRequest = {};
        const pullRequestsByNumber = {};
        for (const release of await this.buildReleases()) {
            pullRequestsByNumber[release.pullRequest.number] = release.pullRequest;
            if (releasesByPullRequest[release.pullRequest.number]) {
                releasesByPullRequest[release.pullRequest.number].push(release);
            }
            else {
                releasesByPullRequest[release.pullRequest.number] = [release];
            }
        }
        const promises = [];
        for (const pullNumber in releasesByPullRequest) {
            promises.push(this.createReleasesForPullRequest(releasesByPullRequest[pullNumber], pullRequestsByNumber[pullNumber]));
        }
        const releases = await Promise.all(promises);
        return releases.reduce((collection, r) => collection.concat(r), []);
    }
    async createReleasesForPullRequest(releases, pullRequest) {
        // create the release
        const promises = [];
        for (const release of releases) {
            promises.push(this.createRelease(release));
        }
        const duplicateReleases = [];
        const githubReleases = [];
        for (const promise of promises) {
            try {
                githubReleases.push(await promise);
            }
            catch (err) {
                if (err instanceof errors_1.DuplicateReleaseError) {
                    logger_1.logger.warn(`Duplicate release tag: ${err.tag}`);
                    duplicateReleases.push(err);
                }
                else {
                    throw err;
                }
            }
        }
        if (duplicateReleases.length > 0 && githubReleases.length === 0) {
            throw duplicateReleases[0];
        }
        // adjust tags on pullRequest
        await Promise.all([
            this.github.removeIssueLabels(this.labels, pullRequest.number),
            this.github.addIssueLabels(this.releaseLabels, pullRequest.number),
        ]);
        return githubReleases;
    }
    async createRelease(release) {
        const githubRelease = await this.github.createRelease(release, {
            draft: release.draft,
            prerelease: release.prerelease,
        });
        // comment on pull request
        const comment = `:robot: Release is at ${githubRelease.url} :sunflower:`;
        await this.github.commentOnIssue(comment, release.pullRequest.number);
        return {
            ...githubRelease,
            path: release.path,
            version: release.tag.version.toString(),
            major: release.tag.version.major,
            minor: release.tag.version.minor,
            patch: release.tag.version.patch,
        };
    }
    async getStrategiesByPath() {
        if (!this._strategiesByPath) {
            logger_1.logger.info('Building strategies by path');
            this._strategiesByPath = {};
            for (const path in this.repositoryConfig) {
                const config = this.repositoryConfig[path];
                logger_1.logger.debug(`${path}: ${config.releaseType}`);
                const strategy = await factory_1.buildStrategy({
                    ...config,
                    github: this.github,
                    path,
                    targetBranch: this.targetBranch,
                });
                this._strategiesByPath[path] = strategy;
            }
        }
        return this._strategiesByPath;
    }
    async getPathsByComponent() {
        if (!this._pathsByComponent) {
            this._pathsByComponent = {};
            const strategiesByPath = await this.getStrategiesByPath();
            for (const path in this.repositoryConfig) {
                const strategy = strategiesByPath[path];
                const component = (await strategy.getComponent()) || '';
                if (this._pathsByComponent[component]) {
                    logger_1.logger.warn(`Multiple paths for ${component}: ${this._pathsByComponent[component]}, ${path}`);
                }
                this._pathsByComponent[component] = path;
            }
        }
        return this._pathsByComponent;
    }
}
exports.Manifest = Manifest;
/**
 * Helper to convert parsed JSON releaser config into ReleaserConfig for
 * the Manifest.
 *
 * @param {ReleaserPackageConfig} config Parsed configuration from JSON file.
 * @returns {ReleaserConfig}
 */
function extractReleaserConfig(config) {
    return {
        releaseType: config['release-type'],
        bumpMinorPreMajor: config['bump-minor-pre-major'],
        bumpPatchForMinorPreMajor: config['bump-patch-for-minor-pre-major'],
        changelogSections: config['changelog-sections'],
        changelogPath: config['changelog-path'],
        releaseAs: config['release-as'],
        skipGithubRelease: config['skip-github-release'],
        draft: config.draft,
        prerelease: config.prerelease,
        draftPullRequest: config['draft-pull-request'],
        component: config['component'],
        packageName: config['package-name'],
        versionFile: config['version-file'],
        extraFiles: config['extra-files'],
        includeComponentInTag: config['include-component-in-tag'],
        changelogType: config['changelog-type'],
        pullRequestTitlePattern: config['pull-request-title-pattern'],
        tagSeparator: config['tag-separator'],
    };
}
/**
 * Helper to convert fetch the manifest config from the repository and
 * parse into configuration for the Manifest.
 *
 * @param {GitHub} github GitHub client
 * @param {string} configFile Path in the repository to the manifest config
 * @param {string} branch Branch to fetch the config file from
 */
async function parseConfig(github, configFile, branch) {
    const config = await github.getFileJson(configFile, branch);
    const defaultConfig = extractReleaserConfig(config);
    const repositoryConfig = {};
    for (const path in config.packages) {
        repositoryConfig[path] = mergeReleaserConfig(defaultConfig, extractReleaserConfig(config.packages[path]));
    }
    const configLabel = config['label'];
    const configReleaseLabel = config['release-label'];
    const manifestOptions = {
        bootstrapSha: config['bootstrap-sha'],
        lastReleaseSha: config['last-release-sha'],
        alwaysLinkLocal: config['always-link-local'],
        separatePullRequests: config['separate-pull-requests'],
        groupPullRequestTitlePattern: config['group-pull-request-title-pattern'],
        plugins: config['plugins'],
        labels: configLabel === undefined ? undefined : [configLabel],
        releaseLabels: configReleaseLabel === undefined ? undefined : [configReleaseLabel],
    };
    return { config: repositoryConfig, options: manifestOptions };
}
/**
 * Helper to parse the manifest versions file.
 *
 * @param {GitHub} github GitHub client
 * @param {string} manifestFile Path in the repository to the versions file
 * @param {string} branch Branch to fetch the versions file from
 */
async function parseReleasedVersions(github, manifestFile, branch) {
    const manifestJson = await github.getFileJson(manifestFile, branch);
    const releasedVersions = {};
    for (const path in manifestJson) {
        releasedVersions[path] = version_1.Version.parse(manifestJson[path]);
    }
    return releasedVersions;
}
/**
 * Find the most recent matching release tag on the branch we're
 * configured for.
 *
 * @param {string} prefix - Limit the release to a specific component.
 * @param {boolean} preRelease - Whether or not to return pre-release
 *   versions. Defaults to false.
 */
async function latestReleaseVersion(github, targetBranch, prefix, pullRequestTitlePattern) {
    var _a;
    const branchPrefix = prefix
        ? prefix.endsWith('-')
            ? prefix.replace(/-$/, '')
            : prefix
        : undefined;
    logger_1.logger.info(`Looking for latest release on branch: ${targetBranch} with prefix: ${prefix}`);
    // collect set of recent commit SHAs seen to verify that the release
    // is in the current branch
    const commitShas = new Set();
    const candidateReleaseVersions = [];
    // only look at the last 250 or so commits to find the latest tag - we
    // don't want to scan the entire repository history if this repo has never
    // been released
    const generator = github.mergeCommitIterator(targetBranch, { maxResults: 250 });
    for await (const commitWithPullRequest of generator) {
        commitShas.add(commitWithPullRequest.sha);
        const mergedPullRequest = commitWithPullRequest.pullRequest;
        if (!mergedPullRequest) {
            continue;
        }
        const branchName = branch_name_1.BranchName.parse(mergedPullRequest.headBranchName);
        if (!branchName) {
            continue;
        }
        // If branchPrefix is specified, ensure it is found in the branch name.
        // If branchPrefix is not specified, component should also be undefined.
        if (branchName.getComponent() !== branchPrefix) {
            continue;
        }
        const pullRequestTitle = pull_request_title_1.PullRequestTitle.parse(mergedPullRequest.title, pullRequestTitlePattern);
        if (!pullRequestTitle) {
            continue;
        }
        const version = pullRequestTitle.getVersion();
        if ((_a = version === null || version === void 0 ? void 0 : version.preRelease) === null || _a === void 0 ? void 0 : _a.includes('SNAPSHOT')) {
            // FIXME, don't hardcode this
            continue;
        }
        if (version) {
            logger_1.logger.debug(`Found latest release pull request: ${mergedPullRequest.number} version: ${version}`);
            candidateReleaseVersions.push(version);
            break;
        }
    }
    // If not found from recent pull requests, look at releases. Iterate
    // through releases finding valid tags, then cross reference
    const releaseGenerator = github.releaseIterator();
    for await (const release of releaseGenerator) {
        const tagName = tag_name_1.TagName.parse(release.tagName);
        if (!tagName) {
            continue;
        }
        if (tagName.component === branchPrefix) {
            logger_1.logger.debug(`found release for ${prefix}`, tagName.version);
            if (!commitShas.has(release.sha)) {
                logger_1.logger.debug(`SHA not found in recent commits to branch ${targetBranch}, skipping`);
                continue;
            }
            candidateReleaseVersions.push(tagName.version);
        }
    }
    logger_1.logger.debug(`found ${candidateReleaseVersions.length} possible releases.`, candidateReleaseVersions);
    if (candidateReleaseVersions.length > 0) {
        // Find largest release number (sort descending then return first)
        return candidateReleaseVersions.sort((a, b) => b.compare(a))[0];
    }
    // If not found from recent pull requests or releases, look at tags. Iterate
    // through tags and cross reference against SHAs in this branch
    const tagGenerator = github.tagIterator();
    const candidateTagVersion = [];
    for await (const tag of tagGenerator) {
        const tagName = tag_name_1.TagName.parse(tag.name);
        if (!tagName) {
            continue;
        }
        if (tagName.component === branchPrefix) {
            if (!commitShas.has(tag.sha)) {
                logger_1.logger.debug(`SHA not found in recent commits to branch ${targetBranch}, skipping`);
                continue;
            }
            candidateTagVersion.push(tagName.version);
        }
    }
    logger_1.logger.debug(`found ${candidateTagVersion.length} possible tags.`, candidateTagVersion);
    // Find largest release number (sort descending then return first)
    return candidateTagVersion.sort((a, b) => b.compare(a))[0];
}
function mergeReleaserConfig(defaultConfig, pathConfig) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
    return {
        releaseType: (_b = (_a = pathConfig.releaseType) !== null && _a !== void 0 ? _a : defaultConfig.releaseType) !== null && _b !== void 0 ? _b : 'node',
        bumpMinorPreMajor: (_c = pathConfig.bumpMinorPreMajor) !== null && _c !== void 0 ? _c : defaultConfig.bumpMinorPreMajor,
        bumpPatchForMinorPreMajor: (_d = pathConfig.bumpPatchForMinorPreMajor) !== null && _d !== void 0 ? _d : defaultConfig.bumpPatchForMinorPreMajor,
        changelogSections: (_e = pathConfig.changelogSections) !== null && _e !== void 0 ? _e : defaultConfig.changelogSections,
        changelogPath: (_f = pathConfig.changelogPath) !== null && _f !== void 0 ? _f : defaultConfig.changelogPath,
        releaseAs: (_g = pathConfig.releaseAs) !== null && _g !== void 0 ? _g : defaultConfig.releaseAs,
        skipGithubRelease: (_h = pathConfig.skipGithubRelease) !== null && _h !== void 0 ? _h : defaultConfig.skipGithubRelease,
        draft: (_j = pathConfig.draft) !== null && _j !== void 0 ? _j : defaultConfig.draft,
        prerelease: (_k = pathConfig.prerelease) !== null && _k !== void 0 ? _k : defaultConfig.prerelease,
        component: (_l = pathConfig.component) !== null && _l !== void 0 ? _l : defaultConfig.component,
        packageName: (_m = pathConfig.packageName) !== null && _m !== void 0 ? _m : defaultConfig.packageName,
        versionFile: (_o = pathConfig.versionFile) !== null && _o !== void 0 ? _o : defaultConfig.versionFile,
        extraFiles: (_p = pathConfig.extraFiles) !== null && _p !== void 0 ? _p : defaultConfig.extraFiles,
        includeComponentInTag: (_q = pathConfig.includeComponentInTag) !== null && _q !== void 0 ? _q : defaultConfig.includeComponentInTag,
        tagSeparator: (_r = pathConfig.tagSeparator) !== null && _r !== void 0 ? _r : defaultConfig.tagSeparator,
        pullRequestTitlePattern: (_s = pathConfig.pullRequestTitlePattern) !== null && _s !== void 0 ? _s : defaultConfig.pullRequestTitlePattern,
    };
}
/**
 * Helper to compare if a list of labels fully contains another list of labels
 * @param {string[]} expected List of labels expected to be contained
 * @param {string[]} existing List of existing labels to consider
 */
function hasAllLabels(expected, existing) {
    const existingSet = new Set(existing);
    for (const label of expected) {
        if (!existingSet.has(label)) {
            return false;
        }
    }
    return true;
}
function commitsAfterSha(commits, lastReleaseSha) {
    if (!commits) {
        return [];
    }
    const index = commits.findIndex(commit => commit.sha === lastReleaseSha);
    if (index === -1) {
        return commits;
    }
    return commits.slice(0, index);
}
//# sourceMappingURL=manifest.js.map