"use strict";
// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseStrategy = void 0;
const manifest_1 = require("../manifest");
const default_1 = require("../versioning-strategies/default");
const default_2 = require("../changelog-notes/default");
const commit_1 = require("../commit");
const version_1 = require("../version");
const tag_name_1 = require("../util/tag-name");
const logger_1 = require("../util/logger");
const pull_request_title_1 = require("../util/pull-request-title");
const branch_name_1 = require("../util/branch-name");
const pull_request_body_1 = require("../util/pull-request-body");
const composite_1 = require("../updaters/composite");
const generic_1 = require("../updaters/generic");
const DEFAULT_CHANGELOG_PATH = 'CHANGELOG.md';
/**
 * A strategy is responsible for determining which files are
 * necessary to update in a release pull request.
 */
class BaseStrategy {
    constructor(options) {
        var _a;
        this.path = options.path || manifest_1.ROOT_PROJECT_PATH;
        this.github = options.github;
        this.packageName = options.packageName;
        this.component =
            options.component || this.normalizeComponent(this.packageName);
        this.versioningStrategy =
            options.versioningStrategy || new default_1.DefaultVersioningStrategy({});
        this.targetBranch = options.targetBranch;
        this.repository = options.github.repository;
        this.changelogPath = options.changelogPath || DEFAULT_CHANGELOG_PATH;
        this.changelogSections = options.changelogSections;
        this.tagSeparator = options.tagSeparator;
        this.skipGitHubRelease = options.skipGitHubRelease || false;
        this.releaseAs = options.releaseAs;
        this.changelogNotes =
            options.changelogNotes || new default_2.DefaultChangelogNotes(options);
        this.includeComponentInTag = (_a = options.includeComponentInTag) !== null && _a !== void 0 ? _a : true;
        this.pullRequestTitlePattern = options.pullRequestTitlePattern;
        this.extraFiles = options.extraFiles || [];
    }
    /**
     * Return the component for this strategy. This may be a computed field.
     * @returns {string}
     */
    async getComponent() {
        if (!this.includeComponentInTag) {
            return '';
        }
        return this.component || (await this.getDefaultComponent());
    }
    async getDefaultComponent() {
        var _a;
        return this.normalizeComponent((_a = this.packageName) !== null && _a !== void 0 ? _a : (await this.getDefaultPackageName()));
    }
    async getPackageName() {
        var _a;
        return (_a = this.packageName) !== null && _a !== void 0 ? _a : (await this.getDefaultPackageName());
    }
    async getDefaultPackageName() {
        var _a;
        return (_a = this.packageName) !== null && _a !== void 0 ? _a : '';
    }
    normalizeComponent(component) {
        if (!component) {
            return '';
        }
        return component;
    }
    /**
     * Override this method to post process commits
     * @param {ConventionalCommit[]} commits parsed commits
     * @returns {ConventionalCommit[]} modified commits
     */
    async postProcessCommits(commits) {
        return commits;
    }
    async buildReleaseNotes(conventionalCommits, newVersion, newVersionTag, latestRelease) {
        var _a;
        return await this.changelogNotes.buildNotes(conventionalCommits, {
            owner: this.repository.owner,
            repository: this.repository.repo,
            version: newVersion.toString(),
            previousTag: (_a = latestRelease === null || latestRelease === void 0 ? void 0 : latestRelease.tag) === null || _a === void 0 ? void 0 : _a.toString(),
            currentTag: newVersionTag.toString(),
            targetBranch: this.targetBranch,
            changelogSections: this.changelogSections,
        });
    }
    async buildPullRequestBody(component, newVersion, releaseNotesBody, _conventionalCommits, _latestRelease) {
        return new pull_request_body_1.PullRequestBody([
            {
                component,
                version: newVersion,
                notes: releaseNotesBody,
            },
        ]);
    }
    /**
     * Builds a candidate release pull request
     * @param {Commit[]} commits Raw commits to consider for this release.
     * @param {Release} latestRelease Optional. The last release for this
     *   component if available.
     * @param {boolean} draft Optional. Whether or not to create the pull
     *   request as a draft. Defaults to `false`.
     * @returns {ReleasePullRequest | undefined} The release pull request to
     *   open for this path/component. Returns undefined if we should not
     *   open a pull request.
     */
    async buildReleasePullRequest(commits, latestRelease, draft, labels = []) {
        const conventionalCommits = await this.postProcessCommits(commit_1.parseConventionalCommits(commits));
        logger_1.logger.info(`Considering: ${conventionalCommits.length} commits`);
        if (conventionalCommits.length === 0) {
            logger_1.logger.info(`No commits for path: ${this.path}, skipping`);
            return undefined;
        }
        const newVersion = await this.buildNewVersion(conventionalCommits, latestRelease);
        const versionsMap = await this.updateVersionsMap(await this.buildVersionsMap(conventionalCommits), conventionalCommits);
        const component = await this.getComponent();
        logger_1.logger.debug('component:', component);
        const newVersionTag = new tag_name_1.TagName(newVersion, this.includeComponentInTag ? component : undefined, this.tagSeparator);
        logger_1.logger.debug('pull request title pattern:', this.pullRequestTitlePattern);
        const pullRequestTitle = pull_request_title_1.PullRequestTitle.ofComponentTargetBranchVersion(component || '', this.targetBranch, newVersion, this.pullRequestTitlePattern);
        const branchName = component
            ? branch_name_1.BranchName.ofComponentTargetBranch(component, this.targetBranch)
            : branch_name_1.BranchName.ofTargetBranch(this.targetBranch);
        const releaseNotesBody = await this.buildReleaseNotes(conventionalCommits, newVersion, newVersionTag, latestRelease);
        if (this.changelogEmpty(releaseNotesBody)) {
            logger_1.logger.info(`No user facing commits found since ${latestRelease ? latestRelease.sha : 'beginning of time'} - skipping`);
            return undefined;
        }
        const updates = await this.buildUpdates({
            changelogEntry: releaseNotesBody,
            newVersion,
            versionsMap,
            latestVersion: latestRelease === null || latestRelease === void 0 ? void 0 : latestRelease.tag.version,
        });
        const updatesWithExtras = composite_1.mergeUpdates(updates.concat(...this.extraFileUpdates(newVersion)));
        const pullRequestBody = await this.buildPullRequestBody(component, newVersion, releaseNotesBody, conventionalCommits, latestRelease);
        return {
            title: pullRequestTitle,
            body: pullRequestBody,
            updates: updatesWithExtras,
            labels,
            headRefName: branchName.toString(),
            version: newVersion,
            draft: draft !== null && draft !== void 0 ? draft : false,
        };
    }
    extraFileUpdates(version) {
        const genericUpdater = new generic_1.Generic({ version });
        return this.extraFiles.map(path => ({
            path: this.addPath(path),
            createIfMissing: false,
            updater: genericUpdater,
        }));
    }
    changelogEmpty(changelogEntry) {
        return changelogEntry.split('\n').length <= 1;
    }
    async updateVersionsMap(versionsMap, conventionalCommits) {
        for (const versionKey of versionsMap.keys()) {
            const version = versionsMap.get(versionKey);
            if (!version) {
                logger_1.logger.warn(`didn't find version for ${versionKey}`);
                continue;
            }
            const newVersion = await this.versioningStrategy.bump(version, conventionalCommits);
            versionsMap.set(versionKey, newVersion);
        }
        return versionsMap;
    }
    async buildNewVersion(conventionalCommits, latestRelease) {
        if (this.releaseAs) {
            logger_1.logger.warn(`Setting version for ${this.path} from release-as configuration`);
            return version_1.Version.parse(this.releaseAs);
        }
        const releaseAsCommit = conventionalCommits.find(conventionalCommit => conventionalCommit.notes.find(note => note.title === 'RELEASE AS'));
        if (releaseAsCommit) {
            const note = releaseAsCommit.notes.find(note => note.title === 'RELEASE AS');
            if (note) {
                return version_1.Version.parse(note.text);
            }
        }
        if (latestRelease) {
            return await this.versioningStrategy.bump(latestRelease.tag.version, conventionalCommits);
        }
        return this.initialReleaseVersion();
    }
    async buildVersionsMap(_conventionalCommits) {
        return new Map();
    }
    async parsePullRequestBody(pullRequestBody) {
        return pull_request_body_1.PullRequestBody.parse(pullRequestBody);
    }
    /**
     * Given a merged pull request, build the candidate release.
     * @param {PullRequest} mergedPullRequest The merged release pull request.
     * @returns {Release} The candidate release.
     */
    async buildRelease(mergedPullRequest) {
        if (this.skipGitHubRelease) {
            logger_1.logger.info('Release skipped from strategy config');
            return;
        }
        if (!mergedPullRequest.sha) {
            logger_1.logger.error('Pull request should have been merged');
            return;
        }
        const pullRequestTitle = pull_request_title_1.PullRequestTitle.parse(mergedPullRequest.title, this.pullRequestTitlePattern) ||
            pull_request_title_1.PullRequestTitle.parse(mergedPullRequest.title, manifest_1.MANIFEST_PULL_REQUEST_TITLE_PATTERN);
        if (!pullRequestTitle) {
            logger_1.logger.error(`Bad pull request title: '${mergedPullRequest.title}'`);
            return;
        }
        const branchName = branch_name_1.BranchName.parse(mergedPullRequest.headBranchName);
        if (!branchName) {
            logger_1.logger.error(`Bad branch name: ${mergedPullRequest.headBranchName}`);
            return;
        }
        const pullRequestBody = await this.parsePullRequestBody(mergedPullRequest.body);
        if (!pullRequestBody) {
            logger_1.logger.error('Could not parse pull request body as a release PR');
            return;
        }
        const component = await this.getComponent();
        let releaseData;
        if (pullRequestBody.releaseData.length === 1 &&
            !pullRequestBody.releaseData[0].component) {
            // standalone release PR, ensure the components match
            if (this.normalizeComponent(branchName.component) !==
                this.normalizeComponent(component)) {
                logger_1.logger.warn(`PR component: ${branchName.component} does not match configured component: ${component}`);
                return;
            }
            releaseData = pullRequestBody.releaseData[0];
        }
        else {
            // manifest release with multiple components
            releaseData = pullRequestBody.releaseData.find(releaseData => {
                return (this.normalizeComponent(releaseData.component) ===
                    this.normalizeComponent(component));
            });
        }
        const notes = releaseData === null || releaseData === void 0 ? void 0 : releaseData.notes;
        if (notes === undefined) {
            logger_1.logger.warn('Failed to find release notes');
        }
        const version = pullRequestTitle.getVersion() || (releaseData === null || releaseData === void 0 ? void 0 : releaseData.version);
        if (!version) {
            logger_1.logger.error('Pull request should have included version');
            return;
        }
        const tag = new tag_name_1.TagName(version, this.includeComponentInTag ? component : undefined, this.tagSeparator);
        const releaseName = component && this.includeComponentInTag
            ? `${component}: v${version.toString()}`
            : `v${version.toString()}`;
        return {
            name: releaseName,
            tag,
            notes: notes || '',
            sha: mergedPullRequest.sha,
        };
    }
    /**
     * Override this to handle the initial version of a new library.
     */
    initialReleaseVersion() {
        return version_1.Version.parse('1.0.0');
    }
    /**
     * Adds a given file path to the strategy path.
     * @param {string} file Desired file path.
     * @returns {string} The file relative to the strategy.
     * @throws {Error} If the file path contains relative pathing characters, i.e. ../, ~/
     */
    addPath(file) {
        // There is no strategy path to join, the strategy is at the root, or the
        // file is at the root (denoted by a leading slash or tilde)
        if (!this.path || this.path === manifest_1.ROOT_PROJECT_PATH || file.startsWith('/')) {
            file = file.replace(/^\/+/, '');
        }
        // Otherwise, the file is relative to the strategy path
        else {
            file = `${this.path.replace(/\/+$/, '')}/${file}`;
        }
        // Ensure the file path does not escape the workspace
        if (/((^|\/)\.{1,2}|^~|^\/*)+\//.test(file)) {
            throw new Error(`illegal pathing characters in path: ${file}`);
        }
        // Strip any trailing slashes and return
        return file.replace(/\/+$/, '');
    }
}
exports.BaseStrategy = BaseStrategy;
//# sourceMappingURL=base.js.map