"use strict";
// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaYoshi = void 0;
const versions_manifest_1 = require("../updaters/java/versions-manifest");
const version_1 = require("../version");
const java_update_1 = require("../updaters/java/java-update");
const base_1 = require("./base");
const changelog_1 = require("../updaters/changelog");
const java_snapshot_1 = require("../versioning-strategies/java-snapshot");
const errors_1 = require("../errors");
const logger_1 = require("../util/logger");
const pull_request_title_1 = require("../util/pull-request-title");
const branch_name_1 = require("../util/branch-name");
const pull_request_body_1 = require("../util/pull-request-body");
const default_1 = require("../versioning-strategies/default");
const java_add_snapshot_1 = require("../versioning-strategies/java-add-snapshot");
const CHANGELOG_SECTIONS = [
    { type: 'feat', section: 'Features' },
    { type: 'fix', section: 'Bug Fixes' },
    { type: 'perf', section: 'Performance Improvements' },
    { type: 'deps', section: 'Dependencies' },
    { type: 'revert', section: 'Reverts' },
    { type: 'docs', section: 'Documentation' },
    { type: 'style', section: 'Styles', hidden: true },
    { type: 'chore', section: 'Miscellaneous Chores', hidden: true },
    { type: 'refactor', section: 'Code Refactoring', hidden: true },
    { type: 'test', section: 'Tests', hidden: true },
    { type: 'build', section: 'Build System', hidden: true },
    { type: 'ci', section: 'Continuous Integration', hidden: true },
];
class JavaYoshi extends base_1.BaseStrategy {
    constructor(options) {
        var _a;
        options.changelogSections = (_a = options.changelogSections) !== null && _a !== void 0 ? _a : CHANGELOG_SECTIONS;
        // wrap the configured versioning strategy with snapshotting
        const parentVersioningStrategy = options.versioningStrategy || new default_1.DefaultVersioningStrategy();
        options.versioningStrategy = new java_snapshot_1.JavaSnapshot(parentVersioningStrategy);
        super(options);
        this.snapshotVersioning = new java_add_snapshot_1.JavaAddSnapshot(parentVersioningStrategy);
    }
    async buildReleasePullRequest(commits, latestRelease, draft, labels = []) {
        if (await this.needsSnapshot()) {
            logger_1.logger.info('Repository needs a snapshot bump.');
            return await this.buildSnapshotPullRequest(latestRelease);
        }
        logger_1.logger.info('No Java snapshot needed');
        return await super.buildReleasePullRequest(commits, latestRelease, draft, labels);
    }
    async buildSnapshotPullRequest(latestRelease) {
        const component = await this.getComponent();
        const newVersion = latestRelease
            ? await this.snapshotVersioning.bump(latestRelease.tag.version, [])
            : this.initialReleaseVersion();
        const versionsMap = await this.buildVersionsMap();
        for (const versionKey of versionsMap.keys()) {
            const version = versionsMap.get(versionKey);
            if (!version) {
                logger_1.logger.warn(`didn't find version for ${versionKey}`);
                continue;
            }
            const newVersion = await this.snapshotVersioning.bump(version, []);
            versionsMap.set(versionKey, newVersion);
        }
        const pullRequestTitle = pull_request_title_1.PullRequestTitle.ofComponentTargetBranchVersion(component || '', this.targetBranch, newVersion);
        const branchName = component
            ? branch_name_1.BranchName.ofComponentTargetBranch(component, this.targetBranch)
            : branch_name_1.BranchName.ofTargetBranch(this.targetBranch);
        const notes = '### Updating meta-information for bleeding-edge SNAPSHOT release.';
        const pullRequestBody = new pull_request_body_1.PullRequestBody([
            {
                component,
                version: newVersion,
                notes,
            },
        ]);
        const updates = await this.buildUpdates({
            newVersion,
            versionsMap,
            changelogEntry: notes,
            isSnapshot: true,
        });
        return {
            title: pullRequestTitle,
            body: pullRequestBody,
            updates,
            labels: [],
            headRefName: branchName.toString(),
            version: newVersion,
            draft: false,
        };
    }
    /**
     * Override this method to post process commits
     * @param {ConventionalCommit[]} commits parsed commits
     * @returns {ConventionalCommit[]} modified commits
     */
    async postProcessCommits(commits) {
        if (commits.length === 0) {
            // For Java commits, push a fake commit so we force a
            // SNAPSHOT release
            commits.push({
                type: 'fake',
                bareMessage: 'fake commit',
                message: 'fake commit',
                breaking: false,
                scope: null,
                notes: [],
                files: [],
                references: [],
                sha: 'fake',
            });
        }
        return commits;
    }
    async needsSnapshot() {
        return versions_manifest_1.VersionsManifest.needsSnapshot((await this.getVersionsContent()).parsedContent);
    }
    async buildVersionsMap() {
        this.versionsContent = await this.getVersionsContent();
        return versions_manifest_1.VersionsManifest.parseVersions(this.versionsContent.parsedContent);
    }
    async getVersionsContent() {
        if (!this.versionsContent) {
            try {
                this.versionsContent = await this.github.getFileContentsOnBranch(this.addPath('versions.txt'), this.targetBranch);
            }
            catch (err) {
                if (err instanceof errors_1.GitHubAPIError) {
                    throw new errors_1.MissingRequiredFileError(this.addPath('versions.txt'), JavaYoshi.name, `${this.repository.owner}/${this.repository.repo}`);
                }
                throw err;
            }
        }
        return this.versionsContent;
    }
    async buildUpdates(options) {
        const updates = [];
        const version = options.newVersion;
        const versionsMap = options.versionsMap;
        updates.push({
            path: this.addPath('versions.txt'),
            createIfMissing: false,
            cachedFileContents: this.versionsContent,
            updater: new versions_manifest_1.VersionsManifest({
                version,
                versionsMap,
            }),
        });
        const pomFilesSearch = this.github.findFilesByFilenameAndRef('pom.xml', this.targetBranch, this.path);
        const buildFilesSearch = this.github.findFilesByFilenameAndRef('build.gradle', this.targetBranch, this.path);
        const dependenciesSearch = this.github.findFilesByFilenameAndRef('dependencies.properties', this.targetBranch, this.path);
        const pomFiles = await pomFilesSearch;
        pomFiles.forEach(path => {
            updates.push({
                path: this.addPath(path),
                createIfMissing: false,
                updater: new java_update_1.JavaUpdate({
                    version,
                    versionsMap,
                    isSnapshot: options.isSnapshot,
                }),
            });
        });
        const buildFiles = await buildFilesSearch;
        buildFiles.forEach(path => {
            updates.push({
                path: this.addPath(path),
                createIfMissing: false,
                updater: new java_update_1.JavaUpdate({
                    version,
                    versionsMap,
                    isSnapshot: options.isSnapshot,
                }),
            });
        });
        const dependenciesFiles = await dependenciesSearch;
        dependenciesFiles.forEach(path => {
            updates.push({
                path: this.addPath(path),
                createIfMissing: false,
                updater: new java_update_1.JavaUpdate({
                    version,
                    versionsMap,
                    isSnapshot: options.isSnapshot,
                }),
            });
        });
        this.extraFiles.forEach(path => {
            updates.push({
                path,
                createIfMissing: false,
                updater: new java_update_1.JavaUpdate({
                    version,
                    versionsMap,
                    isSnapshot: options.isSnapshot,
                }),
            });
        });
        if (!options.isSnapshot) {
            updates.push({
                path: this.addPath(this.changelogPath),
                createIfMissing: true,
                updater: new changelog_1.Changelog({
                    version,
                    changelogEntry: options.changelogEntry,
                }),
            });
        }
        return updates;
    }
    async updateVersionsMap(versionsMap, conventionalCommits) {
        let isPromotion = false;
        const modifiedCommits = [];
        for (const commit of conventionalCommits) {
            if (isPromotionCommit(commit)) {
                isPromotion = true;
                modifiedCommits.push({
                    ...commit,
                    notes: commit.notes.filter(note => !isPromotionNote(note)),
                });
            }
            else {
                modifiedCommits.push(commit);
            }
        }
        for (const versionKey of versionsMap.keys()) {
            const version = versionsMap.get(versionKey);
            if (!version) {
                logger_1.logger.warn(`didn't find version for ${versionKey}`);
                continue;
            }
            if (isPromotion && isStableArtifact(versionKey)) {
                versionsMap.set(versionKey, version_1.Version.parse('1.0.0'));
            }
            else {
                const newVersion = await this.versioningStrategy.bump(version, modifiedCommits);
                versionsMap.set(versionKey, newVersion);
            }
        }
        return versionsMap;
    }
    initialReleaseVersion() {
        return version_1.Version.parse('0.1.0');
    }
}
exports.JavaYoshi = JavaYoshi;
const VERSIONED_ARTIFACT_REGEX = /^.*-(v\d+[^-]*)$/;
const VERSION_REGEX = /^v\d+(.*)$/;
/**
 * Returns true if the artifact should be considered stable
 * @param artifact name of the artifact to check
 */
function isStableArtifact(artifact) {
    const match = artifact.match(VERSIONED_ARTIFACT_REGEX);
    if (!match) {
        // The artifact does not have a version qualifier at the end
        return true;
    }
    const versionMatch = match[1].match(VERSION_REGEX);
    if (versionMatch && versionMatch[1]) {
        // The version is not stable (probably alpha/beta/rc)
        return false;
    }
    return true;
}
function isPromotionCommit(commit) {
    return commit.notes.some(isPromotionNote);
}
function isPromotionNote(note) {
    return note.title === 'RELEASE AS' && note.text === '1.0.0';
}
//# sourceMappingURL=java-yoshi.js.map