"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.RubyYoshi = void 0;
const indent_commit_1 = require("../util/indent-commit");
// Generic
const changelog_1 = require("../updaters/changelog");
// RubyYoshi
const version_rb_1 = require("../updaters/ruby/version-rb");
const base_1 = require("./base");
const fs_1 = require("fs");
const path_1 = require("path");
const manifest_1 = require("../manifest");
const logger_1 = require("../util/logger");
const pull_request_body_1 = require("../util/pull-request-body");
const CHANGELOG_SECTIONS = [
    { type: 'feat', section: 'Features' },
    { type: 'fix', section: 'Bug Fixes' },
    { type: 'perf', section: 'Performance Improvements' },
    { type: 'revert', section: 'Reverts' },
    { type: 'docs', section: 'Documentation' },
    { type: 'style', section: 'Styles', hidden: true },
    { type: 'chore', section: 'Miscellaneous Chores', hidden: true },
    { type: 'refactor', section: 'Code Refactoring', hidden: true },
    { type: 'test', section: 'Tests', hidden: true },
    { type: 'build', section: 'Build System', hidden: true },
    { type: 'ci', section: 'Continuous Integration', hidden: true },
];
class RubyYoshi extends base_1.BaseStrategy {
    constructor(options) {
        var _a;
        super({
            ...options,
            changelogSections: CHANGELOG_SECTIONS,
            commitPartial: fs_1.readFileSync(path_1.resolve(__dirname, '../../../templates/commit.hbs'), 'utf8'),
            headerPartial: fs_1.readFileSync(path_1.resolve(__dirname, '../../../templates/header.hbs'), 'utf8'),
            mainTemplate: fs_1.readFileSync(path_1.resolve(__dirname, '../../../templates/template.hbs'), 'utf8'),
            tagSeparator: '/',
        });
        this.versionFile = (_a = options.versionFile) !== null && _a !== void 0 ? _a : '';
    }
    async buildUpdates(options) {
        const updates = [];
        const version = options.newVersion;
        updates.push({
            path: this.addPath(this.changelogPath),
            createIfMissing: true,
            updater: new changelog_1.Changelog({
                version,
                changelogEntry: options.changelogEntry,
            }),
        });
        const versionFile = this.versionFile
            ? this.versionFile
            : `lib/${(this.component || '').replace(/-/g, '/')}/version.rb`;
        updates.push({
            path: this.addPath(versionFile),
            createIfMissing: false,
            updater: new version_rb_1.VersionRB({
                version,
            }),
        });
        return updates;
    }
    async postProcessCommits(commits) {
        commits.forEach(commit => {
            commit.message = indent_commit_1.indentCommit(commit);
        });
        return commits;
    }
    async buildPullRequestBody(component, newVersion, releaseNotesBody, conventionalCommits, latestRelease) {
        if (!latestRelease) {
            return await super.buildPullRequestBody(component, newVersion, releaseNotesBody, conventionalCommits, latestRelease);
        }
        // summarize the commits that landed:
        let summary = '### Commits since last release:\n\n';
        const updatedFiles = {};
        const repoUrl = `${this.repository.owner}/${this.repository.repo}`;
        for (const commit of conventionalCommits) {
            if (!commit.sha)
                continue;
            const splitMessage = commit.message.split('\n');
            summary += `* [${splitMessage[0]}](https://github.com/${repoUrl}/commit/${commit.sha})\n`;
            if (splitMessage.length > 2) {
                summary = `${summary}<pre><code>${splitMessage
                    .slice(1)
                    .join('\n')}</code></pre>\n`;
            }
            if (commit.files === undefined) {
                logger_1.logger.error('No files for commit - this is likely a bug.');
                continue;
            }
            commit.files.forEach(file => {
                if (this.path === manifest_1.ROOT_PROJECT_PATH || file.startsWith(this.path)) {
                    updatedFiles[file] = true;
                }
            });
        }
        // summarize the files that changed:
        summary = `${summary}\n### Files edited since last release:\n\n<pre><code>`;
        Object.keys(updatedFiles).forEach(file => {
            summary += `${file}\n`;
        });
        summary += `</code></pre>\n[Compare Changes](https://github.com/${repoUrl}/compare/${latestRelease.sha}...HEAD)\n`;
        return new pull_request_body_1.PullRequestBody([
            {
                component,
                version: newVersion,
                notes: releaseNotesBody,
            },
        ], {
            extra: summary,
        });
    }
}
exports.RubyYoshi = RubyYoshi;
//# sourceMappingURL=ruby-yoshi.js.map