import { Octokit } from '@octokit/rest';
import { Repository } from '../repository';
export declare const DEFAULT_FILE_MODE = "100644";
export interface GitHubFileContents {
    sha: string;
    content: string;
    parsedContent: string;
    mode: string;
}
/**
 * This class is a read-through cache aimed at minimizing the
 * number of API requests needed to fetch file data/contents.
 * It lazy-caches data as it reads and will return cached data
 * for resources already fetched.
 */
export declare class RepositoryFileCache {
    private octokit;
    private repository;
    private cache;
    /**
     * Instantiate a new loading cache instance
     *
     * @param {Octokit} octokit An authenticated octokit instance
     * @param {Repository} repository The repository we are fetching data for
     */
    constructor(octokit: Octokit, repository: Repository);
    /**
     * Fetch file contents for given path on a given branch. If the
     * data has already been fetched, return a cached copy.
     *
     * @param {string} path Path to the file
     * @param {string} branch Branch to fetch the file from
     * @returns {GitHubFileContents} The file contents
     */
    getFileContents(path: string, branch: string): Promise<GitHubFileContents>;
}
/**
 * This class is a read-through cache for a single branch aimed
 * at minimizing the number of API requests needed to fetch file
 * data/contents. It lazy-caches data as it reads and will return
 * cached data for resources already fetched.
 */
export declare class BranchFileCache {
    private octokit;
    private repository;
    private branch;
    private cache;
    private treeCache;
    private treeEntries?;
    /**
     * Instantiate a new loading cache instance
     *
     * @param {Octokit} octokit An authenticated octokit instance
     * @param {Repository} repository The repository we are fetching data for
     * @param {string} branch The branch we are fetching data from
     */
    constructor(octokit: Octokit, repository: Repository, branch: string);
    /**
     * Fetch file contents for given path. If the data has already been
     * fetched, return the cached copy.
     *
     * @param {string} path Path to the file
     * @param {string} branch Branch to fetch the file from
     * @returns {GitHubFileContents} The file contents
     */
    getFileContents(path: string): Promise<GitHubFileContents>;
    /**
     * Actually fetch the file contents. Uses the tree API to fetch file
     * data.
     *
     * @param {string} path Path to the file
     */
    private fetchFileContents;
    /**
     * Return the full recursive git tree. If already fetched, return
     * the cached version. If the tree is too big, return null.
     *
     * @returns {TreeEntry[]} The tree entries
     */
    private getFullTree;
    /**
     * Returns the git tree for a given SHA. If already fetched, return
     * the cached version.
     *
     * @param {string} sha The tree SHA
     * @returns {TreeEntry[]} The tree entries
     */
    private getTree;
    /**
     * Fetch the git tree via the GitHub API
     *
     * @param {string} sha The tree SHA
     * @returns {TreeEntry[]} The tree entries
     */
    private fetchTree;
    /**
     * Fetch the git blob from the GitHub API and convert into a
     * GitHubFileContents object.
     *
     * @param {string} blobSha The git blob SHA
     * @param {TreeEntry} treeEntry The associated tree object
     */
    private fetchContents;
}
