"use strict";
// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.BranchFileCache = exports.RepositoryFileCache = exports.DEFAULT_FILE_MODE = void 0;
const logger_1 = require("./logger");
const errors_1 = require("../errors");
exports.DEFAULT_FILE_MODE = '100644';
/**
 * This class is a read-through cache aimed at minimizing the
 * number of API requests needed to fetch file data/contents.
 * It lazy-caches data as it reads and will return cached data
 * for resources already fetched.
 */
class RepositoryFileCache {
    /**
     * Instantiate a new loading cache instance
     *
     * @param {Octokit} octokit An authenticated octokit instance
     * @param {Repository} repository The repository we are fetching data for
     */
    constructor(octokit, repository) {
        this.octokit = octokit;
        this.repository = repository;
        this.cache = new Map();
    }
    /**
     * Fetch file contents for given path on a given branch. If the
     * data has already been fetched, return a cached copy.
     *
     * @param {string} path Path to the file
     * @param {string} branch Branch to fetch the file from
     * @returns {GitHubFileContents} The file contents
     */
    async getFileContents(path, branch) {
        let fileCache = this.cache.get(branch);
        if (!fileCache) {
            fileCache = new BranchFileCache(this.octokit, this.repository, branch);
            this.cache.set(branch, fileCache);
        }
        return await fileCache.getFileContents(path);
    }
}
exports.RepositoryFileCache = RepositoryFileCache;
/**
 * This class is a read-through cache for a single branch aimed
 * at minimizing the number of API requests needed to fetch file
 * data/contents. It lazy-caches data as it reads and will return
 * cached data for resources already fetched.
 */
class BranchFileCache {
    /**
     * Instantiate a new loading cache instance
     *
     * @param {Octokit} octokit An authenticated octokit instance
     * @param {Repository} repository The repository we are fetching data for
     * @param {string} branch The branch we are fetching data from
     */
    constructor(octokit, repository, branch) {
        this.octokit = octokit;
        this.repository = repository;
        this.branch = branch;
        this.cache = new Map();
        this.treeCache = new Map();
    }
    /**
     * Fetch file contents for given path. If the data has already been
     * fetched, return the cached copy.
     *
     * @param {string} path Path to the file
     * @param {string} branch Branch to fetch the file from
     * @returns {GitHubFileContents} The file contents
     */
    async getFileContents(path) {
        const cached = this.cache.get(path);
        if (cached) {
            return cached;
        }
        const fetched = await this.fetchFileContents(path);
        this.cache.set(path, fetched);
        return fetched;
    }
    /**
     * Actually fetch the file contents. Uses the tree API to fetch file
     * data.
     *
     * @param {string} path Path to the file
     */
    async fetchFileContents(path) {
        // try to use the entire git tree if it's not too big
        const treeEntries = await this.getFullTree();
        if (treeEntries) {
            logger_1.logger.debug(`Using full tree to find ${path}`);
            const found = treeEntries.find(entry => entry.path === path);
            if (found === null || found === void 0 ? void 0 : found.sha) {
                return await this.fetchContents(found.sha, found);
            }
            throw new errors_1.FileNotFoundError(path);
        }
        // full tree is too big, use data API to fetch
        const parts = path.split('/');
        let treeSha = this.branch;
        let found;
        for (const part of parts) {
            const tree = await this.getTree(treeSha);
            found = tree.find(item => item.path === part);
            if (!(found === null || found === void 0 ? void 0 : found.sha)) {
                throw new errors_1.FileNotFoundError(path);
            }
            treeSha = found.sha;
        }
        if (found === null || found === void 0 ? void 0 : found.sha) {
            return await this.fetchContents(found.sha, found);
        }
        throw new errors_1.FileNotFoundError(path);
    }
    /**
     * Return the full recursive git tree. If already fetched, return
     * the cached version. If the tree is too big, return null.
     *
     * @returns {TreeEntry[]} The tree entries
     */
    async getFullTree() {
        if (this.treeEntries === undefined) {
            // fetch all tree entries recursively
            const { data: { tree, truncated }, } = await this.octokit.git.getTree({
                owner: this.repository.owner,
                repo: this.repository.repo,
                tree_sha: this.branch,
                recursive: 'true',
            });
            if (truncated) {
                // the full tree is too big to use, mark it as unusable
                this.treeEntries = null;
            }
            else {
                this.treeEntries = tree;
            }
        }
        return this.treeEntries;
    }
    /**
     * Returns the git tree for a given SHA. If already fetched, return
     * the cached version.
     *
     * @param {string} sha The tree SHA
     * @returns {TreeEntry[]} The tree entries
     */
    async getTree(sha) {
        const cached = this.treeCache.get(sha);
        if (cached) {
            return cached;
        }
        const fetched = await this.fetchTree(sha);
        this.treeCache.set(sha, fetched);
        return fetched;
    }
    /**
     * Fetch the git tree via the GitHub API
     *
     * @param {string} sha The tree SHA
     * @returns {TreeEntry[]} The tree entries
     */
    async fetchTree(sha) {
        const { data: { tree }, } = await this.octokit.git.getTree({
            owner: this.repository.owner,
            repo: this.repository.repo,
            tree_sha: sha,
            recursive: 'false',
        });
        return tree;
    }
    /**
     * Fetch the git blob from the GitHub API and convert into a
     * GitHubFileContents object.
     *
     * @param {string} blobSha The git blob SHA
     * @param {TreeEntry} treeEntry The associated tree object
     */
    async fetchContents(blobSha, treeEntry) {
        const { data: { content }, } = await this.octokit.git.getBlob({
            owner: this.repository.owner,
            repo: this.repository.repo,
            file_sha: blobSha,
        });
        return {
            sha: blobSha,
            mode: treeEntry.mode || exports.DEFAULT_FILE_MODE,
            content,
            parsedContent: Buffer.from(content, 'base64').toString('utf8'),
        };
    }
}
exports.BranchFileCache = BranchFileCache;
//# sourceMappingURL=file-cache.js.map